#!/bin/bash -l

BASEDIR="/build/nightly"
LOCKFILE="/tmp/.nightly-builds.lock"

UPLOAD_DIR="${BASEDIR}/upload"
UPLOAD_TO="pakfire@git.ipfire.org:/pub/nightly"
RSYNC_ARGS=""

extract_installer_from_iso() {
	local dir="${1}"

	local tmpdir="$(mktemp -d)"
	mkdir -p "${dir}/images"

	local file
	for file in ${dir}/*.iso; do
		if mount -o loop,ro "${file}" "${tmpdir}"; then
			local f
			for f in vmlinuz instroot; do
				cp -f "${tmpdir}/boot/isolinux/${f}" "${dir}/images"
			done
			umount "${tmpdir}"

			local ext
			for ext in "" ".md5"; do
				ln -s --relative "${file}${ext}" \
					"${dir}/images/installer.iso${ext}"
			done
			break
		fi
	done 2>/dev/null

	rm -rf "${tmpdir}"
}

build() {
	local dir="${1}"
	shift

	# Check if this actually source of IPFire 2
	[ -x "${dir}/make.sh" ] || return 1

	pushd "${dir}"

	local branch="$(git config build.branch)"
	[ -z "${branch}" ] && branch="master"

	local commit_old="$(git rev-parse HEAD)"
	git fetch

	# Search for the latest core branch
	if [ "${branch}" = "core" ]; then
		branch=$(git branch -r | awk -F/ '{ print $NF }' | grep ^core | sort --version-sort | tail -n1)
	fi

	# If the branch was not updated, we won't build
	local commit_new="$(git rev-parse origin/${branch})"
	[ "${commit_old}" = "${commit_new}" ] && return 2

	# Checkout the latest commit
	git reset --hard "${commit_new}"

	local now="$(git log --format="%ci" -1 "${commit_new}")"

	local targets="$(git config build.targets)"
	[ -z "${targets}" ] && targets="i586"

	local target
	for target in ${targets}; do
		local build_path="${UPLOAD_DIR}/${branch}/${now}-${commit_new:0:8}"
		local build="${build_path}/${target}"

		# Ready for build
		./make.sh --target="${target}" clean

		# Download the toolchain if required
		./make.sh --target="${target}" gettoolchain || continue

		# Download all sources
		./make.sh --target="${target}" downloadsrc || continue

		# Execute the build
		mkdir -p "${build}"
		./make.sh --target="${target}" build | tee "${build}/build.log"
		local ret=${PIPESTATUS[0]}

		# Save the changelog
		git log -50 > "${build}/changelog.txt"

		# Save the result
		if [ "${ret}" = "0" ]; then
			mv -v *.iso *.img.gz *.img.xz *.tar.bz2 *.md5 packages/ "${build}"
			extract_installer_from_iso "${build}"
			touch "${build}/.success"
		fi
		mv -v log/ "${build}"

		# Cleanup the build environment
		./make.sh --target="${target}" clean

		# Upload the result
		# If that failed, we will keep the result and it will
		# be retried with the next build. If that succeeded, the
		# build will be removed from disk.
		if sync; then
			rm -rf "${UPLOAD_DIR}"
		fi
	done

	popd
}

sync() {
	mkdir -p "${UPLOAD_DIR}"

	rsync -avHz --progress ${RSYNC_ARGS} \
		"${UPLOAD_DIR}/" "${UPLOAD_TO}"
}

is_locked() {
	[ -e "${LOCKFILE}" ]
}

lock() {
	touch "${LOCKFILE}"
}

unlock() {
	rm -f "${LOCKFILE}"
}


# Don't start again if the script is already running
# or if an other build script is running
if is_locked || pgrep make.sh >/dev/null; then
	exit 0
fi

# Lock
trap unlock EXIT
lock

for repo in $(find ${BASEDIR} -maxdepth 3 -type d -name ".git"); do
	[ -d "${repo}" ] || continue

	build "$(dirname ${repo})"
done

exit 0
